# -*- coding: utf-8 -*-

from paver.easy import info, debug, task, consume_args

from sett import which, ROOT, optional_import
from sett.watcher import Watcher
from sett.utils import task_name
from sett.utils.dispatch import Dispatcher

from http.server import HTTPServer, SimpleHTTPRequestHandler

sphinx = optional_import('sphinx')


class SphinxDispatcher(Dispatcher):
    def __init__(self, builder):
        self.builder = builder

    @Dispatcher.auto
    @Dispatcher.on('watch', -1)
    def compile(self, paths=None):
        """Launches the compilation of the doc"""
        self.builder()

    def serve(self, port=8000):
        watcher = self.get_watcher()
        http = HTTPServer(('localhost', int(port)), SimpleHTTPRequestHandler)
        with watcher:
            info('Start serving doc on http://localhost:%s/', port)
            try:
                http.serve_forever()
            except KeyboardInterrupt:
                info('Stopping')
                return

    def get_watcher(self):
        debug('Building/watching with %s', self.builder)
        return Watcher(self.builder, self.builder.paths, {'.py', '.md', '.rst'})

    def watch(self):
        """Watches the sass source directories and launches compilation as needed"""
        watcher = self.get_watcher()
        info('Starting watch %s', watcher)
        try:
            watcher()
        except KeyboardInterrupt:
            pass


class Sphinx(object):
    def __init__(self, in_doc_path, out_doc_path):
        self.in_doc_path = in_doc_path
        self.out_doc_path = out_doc_path

    def __repr__(self):
        return 'Sphinx({!r}, {!r})'.format(self.in_doc_path, self.out_doc_path)

    @property
    def paths(self):
        return [
            ROOT,
            self.in_doc_path,
        ]

    def __call__(self, filenames=None):
        debug('Building %s', filenames)
        cmd = [which.search('sphinx-build'), self.in_doc_path, self.out_doc_path]
        if filenames:
            cmd.extend(filenames)
        sphinx.build_main(cmd)


@task
@consume_args
@task_name('sphinx')
def sphinx_dispatch(ROOT, args, defaults):
    """
    Usage: sphinx [compile|watch|serve]
    """
    souce = ROOT.joinpath(defaults.SPHINX_SOURCE)
    outdir = ROOT.joinpath(defaults.SPHINX_OUTDIR)

    sphinx_generator = Sphinx(souce, outdir)
    sd = SphinxDispatcher(sphinx_generator)
    sd(*args)
